; ******************************************************************************************************************
; **  REALIZADO POR FADISHOP  Tarjeta: FADICLOCK  	http://www.fadishop.eu	FADITECO, S.L.U. 	Lleonard Garcia	**
; **  Este programa lee la temperatura cada x segundos, calcula el promedio cada minuto y lo almacena en la SRAM  **
; ** 	Activadas alarma 1(cada segundo) y alarma2 (cada minuto) connectadas a interrupcin en INT2 (B.2)      	**
; ** 	LA DIRECCIN DEL SLAVE I2C DS3232 ES FIJA $D0.					      					**
; **  ENCAMINAMIENTO INTERRUPCIN ALARMAS	JP6(JUMPER AZUL)	(I0=1, I1=0, I2=1, portC.7=0;  Rpullup=1)			** 
; ******************************************************************************************************************
; CONFIGURACIONES
#picaxe 28x2		
let dirsB=%10000000	; 1=salida 0=entrada
let dirsC=%00000000	; 1=salida 0=entrada
; C.3 I2C_SCL
; C.4 I2C_SDA
;adcsetup = %00000000 	' CONFIGURACIN ENTRADAS ANALGICAS
;setfreq em16		; Resonador/oscilador externo a 16Mhz.
device_DS3232SN:
symbol adress_slave_A =$D0	; Direccin fija I2C del DS3232 $D0
symbol @ahora=$00			; direccin interna de segundos, minutos y hora.
symbol @hoy=$03			; direccin interna de dia_semanal, dia, siglo-mes y ao.
symbol @alm1=$07			; direccin interna de alarma_1(seg, min, hora y dia_sem/dia).
symbol @alm2=$0B			; direccin interna de alarma_2(min, hora y dia_sem/dia).
symbol @control=$0E		; (7)/EOSC (6)BBSQW (5)CONV (4-3)RS2-1 (2)INTCN (1)A2IE (0)A1IE
symbol @status=$0F		; (7)OSF (6)BB32KHZ (5-4)CRATE (3)EN32KHZ (2)BSY (1)A2F (0)A1F
symbol @offset=$10		; offset temperatura. (aade/sustrae capacitancia)
symbol @temp=$11			; direccin word interna de MSB_temperatura y LSB_temperatura.
symbol @test=$13			; reservada par test.
symbol @sram=$14			; direccin interna del inicio del bloque de 235 bytes SRAM.
symbol control=b16		; variable control DS3232.
symbol status=b18			; variable control/status DS3232.

; DEFINICIONES
symbol segundos=b0
symbol minutos=b1
symbol hora=b2			
symbol dia_sem=b3
symbol dia=b4
symbol mes=b5
symbol anno=b6
symbol T_msb=b7
symbol T_lsb=b8

symbol acumMsb=w19
symbol acumLsb=w20
symbol medMinMsb=w21
symbol medMinLsb=w22
symbol N_muestra=b48
symbol @SramMin=$20					; direccin de la Sram donde se almacenan los promedios

;INICIALITZACIONES:
let control =%00011100					; coincide con inicializacin por defecto.
let status =%11001000					; coincide con inicializacin por defecto.
call fijar_fecha_hora					; modifica la fecha-hora del RTC DS3232.
call fijar_alarma1					; fija la alarma_1 del RTC DS3232.
call fijar_alarma2					; fija la alarma_1 del RTC DS3232.
configuracio_interrupcions:
hint2flag = 0						; borrar el flag INT_2.
symbol hint_setting = %00000100 			; se activan las interrupciones por INT2.
hintsetup hint_setting					; transfiere dicha activacin.
setintflags %00000100,%00000100 			; se habilitan las interrupciones INT2 per flanco descendente.

main:
call	solicitar_temperatura
i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1)
readi2c @ahora,(segundos,minutos,hora,dia_sem)	; lectura hora actual.
'readi2c @hoy,(dia,mes,anno)				; lectura fecha actual.
readi2c @temp,(T_msb,T_lsb)				; lectura temperatura.
'readi2c @alm1,(b10,b11,b12,b13)			; lectura alarma_1.
'readi2c @control,(b17,b19)				; lectura registres
;readi2c INICI,(b0,b1)
debug
PAUSE 2000
goto main


; SECUENCIA DE INTERRUPCIN
interrupt:
readi2c @status,(status)				; Se lee los registros de control y estado.
let b29=status and %00000001
if b29!=0 then call alarma1				; S A1F=1 ejecuta alarma1
let b29=status and %00000010
if b29!=0 then call alarma2				; S A1F=1 ejecuta alarma2
hint2flag = 0						; se borra el flag INT1 y INT2
hintsetup hint_setting					; se vuelven a activar las interrupciones por INT2.
setintflags %00000100,%00000100 			; se habilitan las interrupciones INT1 y INT2 per flanco descendente.
return							; despus de return es cuando realmente habilita la interrupcin

alarma1:
let status=status and %11111110			; Se borra el flag de alarma_1 en DS3232.
writei2c @status,(status)				; Se envia habilitacin y borrado de flag A1F.
call	annadir_muestra
return


alarma2:
let status=status and %11111101			; Se borra el flag de alarma_2 en DS3232.
writei2c @status,(status)				; Se envia habilitacin y borrado de flag A2F
call promediar_segundos
let acumMsb=0
let acumLsb=0
let N_muestra=0

let b50=@SramMin
let b51=bcdtobin minutos
let b51=b51*2
let b50=b50 + b51 
writei2c b50,(medMinMsb,medMinLsb)			
readi2c @SramMin,(b52,b53,b54,b55)
return


fijar_fecha_hora:
symbol xseg=b30
symbol xminu=b31
symbol xhora=b32
symbol xsemdia=b33
symbol xdia=b34
symbol xmes=b35
symbol xanno=b36
let xseg=$25						; @_interna DS3232: $00
let xminu=$00						; @_interna DS3232: $01
let xhora=$13						; @_interna DS3232: $02
let xsemdia=$00						; @_interna DS3232: $03
let xdia=$03						; @_interna DS3232: $04
let xmes=$03						; @_interna DS3232: $05
let xanno=$12						; @_interna DS3232: $06
low xhora,6							; LOW: 24H          HIGH: AM/PM
i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1)
writei2c @ahora,(xseg,xminu,xhora,xsemdia,xdia,xmes,xanno); Se envia fecha-hora actual al RTC DS3232
return


fijar_alarma1:
symbol a1_seg=b30
symbol a1_min=b31
symbol a1_hor=b32						
symbol a1_dia_sem=b33
symbol a1_modo=b34
let a1_seg=$00
let a1_min=$01
let a1_hor=$13						; Formato hora->bit(6)=0:24h, bit(6)=1:AM/PM 12h.
let a1_dia_sem=$03					; Formato dia-> bit(6)=0:dia del mes, bit(6)=1:dia de la semana.
let a1_modo=4
low a1_hor,6						; LOW: 24H          HIGH: AM/PM
low a1_dia_sem,6						; LOW: dia del mes  HIGH: dia de la semana.						;
select case a1_modo
case 0 
clearbit a1_seg,7  clearbit a1_min,7  clearbit a1_hor,7   clearbit a1_dia_sem,7 	;0: una sla vez(dia,hora,minuto,segundo).
case 1 
clearbit a1_seg,7  clearbit a1_min,7  clearbit a1_hor,7   setbit a1_dia_sem,7		;1: en hora minuto y segundo.
case 2 
clearbit a1_seg,7  clearbit a1_min,7  setbit a1_hor,7  setbit a1_dia_sem,7		;2: en minuto y segundo.
case 3 
clearbit a1_seg,7  setbit a1_min,7 setbit a1_hor,7  setbit a1_dia_sem,7			;3: en segundo.
case 4 
setbit a1_seg,7 setbit a1_min,7 setbit a1_hor,7  setbit a1_dia_sem,7			;4: generador peridico de 1 segundo.
end select

i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1).
writei2c @alm1,(a1_seg,a1_min,a1_hor,a1_dia_sem); Se envia la configuracin de la alarma_1.
let control=control or %00000001			; Se habilita alarma_1 por interrupcin A1E.
let control=control or %00000100			; Se habilita interrupcin por alarmas INTCN en DS3232.
let status=status and %11111110			; Se borra el flag de alarma_1 en DS3232.
writei2c @control,(control,status)			; Se envia habilitacin y borrado de flag A1F.
return

fijar_alarma2:
symbol a2_min=b31
symbol a2_hor=b32						
symbol a2_dia_sem=b33
symbol a2_modo=b34
let a2_min=$01
let a2_hor=$13						; Formato hora->bit(6)=0:24h, bit(6)=1:AM/PM 12h.
let a2_dia_sem=$03					; Formato dia-> bit(6)=0:dia del mes, bit(6)=1:dia de la semana.
let a2_modo=4
low a2_hor,6						; LOW: 24H          HIGH: AM/PM
low a2_dia_sem,6						; LOW: dia del mes  HIGH: dia de la semana.						;
select case a2_modo
case 0 
clearbit a2_min,7  clearbit a2_hor,7   clearbit a2_dia_sem,7 	;0: una sla vez(dia,hora,minuto).
case 1 
clearbit a2_min,7  clearbit a2_hor,7   high a2_dia_sem,7		;1: en hora y minuto.
case 2 
clearbit a2_min,7  setbit a2_hor,7  setbit a2_dia_sem,7		;2: en minuto 
case 3 
setbit a2_min,7 setbit a2_hor,7  setbit a2_dia_sem,7			;3: generador peridico de 1 minuto
case 4 
setbit a2_min,7 setbit a2_hor,7  setbit a2_dia_sem,7			;4: generador peridico de 1 minuto
end select


i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1)
writei2c @alm2,(a2_min,a2_hor,a2_dia_sem)		; Se envia la configuracin de la alarma_2.
let control=control or %00000010			; Se habilita alarma_1 por interrupcin A2E.
let control=control or %00000100			; Se habilita interrupcin por alarmas INTCN en DS3232.
let status=status and %11111101			; Se borra el flag de alarma_2 en DS3232.
writei2c @control,(control,status)			; Se envia habilitacin y borrado de flag A2F.
return


annadir_muestra:
acumMsb=T_msb+acumMsb
acumLsb=T_lsb+acumLsb
inc N_muestra
return

promediar_segundos:
medMinMsb=acumMsb/N_muestra
medMinLsb=acumLsb/N_muestra
return

solicitar_temperatura:
i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1)
readi2c @status,(b10)					; Lee estado del DS3232
let b10=b10 and %00000100				; Est efectuando una conversin (BSY=1)?
if b10<>0 then return					; (SI=1) No hacer nada, volver.
          else let control=control or %00100000	; (NO=0) Forzar conversin de temperatura CONV.
			i2cslave adress_slave_A, i2cslow, i2cbyte		; Direccin I2C del DS3232 (0XD0-0XD1)
		   	writei2c @control,(control)	
endif
return
